;==========================================================
; TCIRCLE.LSP Copyright 1994 by Looking Glass Microproducts
;==========================================================
; Draw circle tangent at point 
;==========================================================
(defun C:TCIRCLE (/ ERROR OLD-ERROR PUSHVARS POPVARS NOTRANS 
                 BITSET SHOW-OSMODE XGETDIST CIRCLE TCIRCLE)

   ;=======================================================
   ; Error Handler
   ;=======================================================
   (defun ERROR (S)
      (if (not
             (member
                S
                '("Function cancelled" "console break")
             )
          )
         (princ S)
      )
      (POPVARS)
      (command ".undo" "end" ".undo" 1)
      (princ)
   )
   ;=======================================================
   ; Set and Save System Variables
   ;=======================================================
   (defun PUSHVARS (VLIST)
      (foreach PAIR VLIST
         (setq
            SYSVARS (cons
                       (cons
                          (strcase (car PAIR))
                          (getvar
                             (car PAIR)
                          )
                       )
                       SYSVARS
                    )
         )
         (if (cdr PAIR)
            (command ".setvar" (car PAIR) (cdr PAIR))
         )
      )
   )
   ;=======================================================
   ; Restore System Variables
   ;=======================================================
   (defun POPVARS ()
      (foreach PAIR SYSVARS
         (command ".setvar" (car PAIR) (cdr PAIR))
      )
      (setq *error* OLD-ERROR)
      (setq SYSVARS nil)
   )
   ;=======================================================
   ; Disallow transparent invocation of routine.
   ;=======================================================
   (defun NOTRANS ()
      (cond
         ((zerop (logand (getvar "cmdactive") 1)))
         ((alert
           "This command may not be invoked transparently."
          )
         )
      )
   )
   ;=======================================================
   ; Bit Set
   ;=======================================================
   (defun BITSET (A B) (/= 0 (logand A B)))
   ;=======================================================
   ; Show Osmode
   ;=======================================================
   (defun SHOW-OSMODE (/ OSMODE OSMODES TMODES)
       (setq
         OSMODES '((1 . "Endpoint")
                    (2 . "Midpoint")
                    (4 . "Center")
                    (8 . "Node")
                    (16 . "Quadrant")
                    (32 . "Intersection")
                    (64 . "Insertion")
                    (128 . "Perpendicular")
                    (256 . "Tangent")
                    (512 . "Nearest")
                    (1024 . "Quick")
                 )
      )
      (setq OSMODE (getvar "osmode") TMODES "")
      (foreach ENTRY OSMODES
         (if (BITSET OSMODE (car ENTRY))
            (progn
               (if (/= "" TMODES)
                  (setq TMODES (strcat TMODES ", "))
               )
               (setq TMODES (strcat TMODES (cdr ENTRY)))
            )
         )
      )
      TMODES
   )
   ;=======================================================
   ; Get Distance With Default
   ;=======================================================
   (defun XGETDIST (PT PRMPT DEFAULT)
      (cond
         ((getdist
             PT
             (strcat
                PRMPT
                (if DEFAULT
                   (strcat " <" (rtos DEFAULT) ">")
                   ""
                )
                ": "
             )
          )
         )
         (DEFAULT)
      )
   )
   ;=======================================================
   ; Draw Circle at p1 with center perpdir from pt
   ;======================================================= 
   (defun CIRCLE (P1 PERPDIR / DEFRAD DIAM)
      (setvar "osmode" 0)
      (if (>= PERPDIR pi)
         (setq PERPDIR (- PERPDIR pi))
      )
      (setq DEFRAD (getvar "circlerad"))
      (if (zerop DEFRAD)
         (setq DEFRAD nil)
      )
      (initget "Radius" (if DEFRAD 6 7))
      (setq
         DIAM (XGETDIST
                 P1
                 "\nRadius/<Diameter>"
                 (if DEFRAD
                    (* 2.0 DEFRAD)
                 )
              )
      )
      (if (= DIAM "Radius")
         (progn
            (initget (if DEFRAD 6 7))
            (setq
               DIAM (* 2.0 (XGETDIST P1 "\nRadius" DEFRAD))
            )
         )
      )
      (setvar "blipmode" 0)
      (prompt
         "\nPICK to mirror, or RETURN to accept: "
      )
      (while (progn
                (setq P2 (polar P1 PERPDIR DIAM))
                (command
                   ".circle" "2p" "none" P1 "none" P2
                )
                (redraw (entlast) 3)
                (getpoint)
             )
         (entdel (entlast))
         (setq DIAM (- DIAM))
      )
      (redraw (entlast) 4)
      (setvar "lastpoint" P1)
   )
   ;=======================================================
   ; Main Routine
   ;=======================================================
   (defun TCIRCLE (/ PRMPT)
      (if (= 0 (getvar "osmode"))
         (setvar "osmode" 512) ; nearest
      )
      (setq
         PRMPT (strcat
                  "\nTangency point <"
                  (SHOW-OSMODE)
                  ">: "
               )
      )
      (cond
         ((null (setq P0 (getpoint PRMPT))))
         ((null (setq P1 (osnap P0 "nea")))
            (princ
               "\nNo tangency at that point."
            )
         )
         ((progn
             (setvar "aperture" 1)
             (setq P2 (osnap P1 "cen"))
          )
            (CIRCLE P1 (angle P2 P1))
         )
         ((and
             (setq P3 (osnap P1 "mid"))
             (setq P4 (osnap P1 "end"))
          )
            (CIRCLE P1 (+ (angle P3 P4) (* 0.5 pi)))
         )
         (t
            (princ
               "\nNo tangency at that point."
            )
         )
      )
   )
   ;==========================================================
   ; Body of c:tcircle  
   ;==========================================================
   (if (NOTRANS)
      (progn
         (setq OLD-ERROR *error* *error* ERROR)
         (setvar "cmdecho" 0)
         (command ".undo" "group")
         (PUSHVARS
            '(("osmode") ("blipmode") ("aperture"))
         )
         (TCIRCLE)
         (POPVARS)
         (command ".undo" "end")
         (setq *error* OLD-ERROR)
      )
   )
   (princ)
)
(princ
   (strcat
      "  TCIRCLE.LSP v1.0 (Copyright 1994 by "
      "Looking Glass Microproducts) loaded."
   )
)
(princ)

